# © MNELAB developers
#
# License: BSD (3-clause)

from .annotations import AnnotationsDialog
from .append import AppendDialog
from .calc import CalcDialog
from .channel_properties import ChannelPropertiesDialog
from .crop import CropDialog
from .drop_bad_epochs import DropBadEpochsDialog
from .epoch import EpochDialog
from .erds import ERDSDialog, ERDSTopomapsDialog
from .error_message import ErrorMessageBox
from .events import EventsDialog
from .filter import FilterDialog
from .find_events import FindEventsDialog
from .history import HistoryDialog
from .interpolate_bads import InterpolateBadsDialog
from .mat import MatDialog
from .montage import MontageDialog
from .pick_channels import PickChannelsDialog
from .plot_evoked import PlotEvokedComparisonDialog, PlotEvokedDialog, PlotEvokedTopomaps
from .reference import ReferenceDialog
from .rename_channels import RenameChannelsDialog
from .run_ica import RunICADialog
from .xdf_chunks import XDFChunksDialog
from .xdf_metadata import XDFMetadataDialog
from .xdf_streams import XDFStreamsDialog
